// This example shows out the inheritance concept of C++

#include "vector.hpp"

// inheritance attribute modifiers
// 
// Father      modifier   Child
// private   * public     = n.a.
// protected * public     = protected
// public    * public     = public
// 
// private   * protected  = n.a.
// protected * protected  = protected
// public    * protected  = protected
//
// private   * private    = n.a.
// protected * private    = private
// public    * private    = private

class VectorInf : public Vector
{
public:
    // constructor: just call the father's one.
    // after ":" comes the initialization list                   
    // where father's constructor can be called                  
    // along with the correct initialization of reference objects
    // belonging to the class (if any)                           
    VectorInf(const unsigned int nelem) : Vector(nelem)
    {                
        fprintf(stdout,"VectorInf class: calling parameters_list-constructor\n");
    }                                                   
                                       
    // copy constructor from a VectorInf object
    VectorInf(const VectorInf &v) : Vector(v.getN())
    {
        fprintf(stdout,"VectorInf class: calling copy-constructor from a VectorInf object\n");

        for (unsigned int i=0; i<N; i++)
            getElem(i)=v.getElem(i);
    }

    // copy constructor from a Vector object
    VectorInf(const Vector &v): Vector(v.getN())
    {
        fprintf(stdout,"VectorInf class: calling copy-constructor from a Vector object\n");

        for (unsigned int i=0; i<N; i++)
            getElem(i)=v.getElem(i);
    }

    // method to implement the element-wise multiplication
    VectorInf prod(const VectorInf &v) const
    {
        VectorInf res=*this;

        if (v.getN()==N)
            for (unsigned int i=0; i<N; i++)
                res.getElem(i)=getElem(i)*v.getElem(i);

        return res;
    }

    // method to implement infinity norm
    float norm() const
    {
        float res=0.0;

        // scroll the vector and take the maximum
        // component in absolute value
        for (unsigned int i=0; i<N; i++)
        {
            float tmp=abs(getElem(i));

            if (tmp>res)
                res=tmp;
        }

        return res;
    }
};


int main()
{
    // definitions
    VectorInf v1(3), v2(3);

    // fill up vectors components
    v1.getElem(0)=1.0F;  v2.getElem(0)=0.0F;
    v1.getElem(1)=2.0F;  v2.getElem(1)=1.0F;
    v1.getElem(2)=3.0F;  v2.getElem(2)=0.0F;

    fprintf(stdout,"first vector components\n");
    v1.print();

    fprintf(stdout,"second vector components\n");
    v2.print();

    // execute algebric operations on vectors
    VectorInf w1=v1.add(v2);
    // the sequence of calls for the previous instruction is as follows:
    // well, we expect a couple of Vector copy-constructor/destructor for the res object internally to add();
    // then, we expect to read the print out of VectorInf copy-constructor that in turn (so just beforehand)
    // calls the Vector parameters_list-construtor; that's it!
    // this is what happens in release mode; by trying it out in debug mode, for some platform as Windows,
    // you'll find out that a further temporary object is created and disposed in the main scope.
    // now you should be aware/afraid of how tangled may be the C++ :) ... but don't give up!!!

    fprintf(stdout,"********\n");
    VectorInf w2=v1.prod(v2);
    // try to figure out by yourself what's going on for w2 :)

    float norm=w1.norm();

    // dump results
    fprintf(stdout,"sum\n");
    w1.print();

    fprintf(stdout,"element-wise product\n");
    w2.print();

    fprintf(stdout,"infinity norm\n");
    fprintf(stdout,"%.3f\n",norm);

    return 0;
}

